@@ -1,6 +1,7 @@
 #include "config.h"
 
 #include <libgnome/libgnome.h>
+#include <vicious.h>
 #include "gdm.h"
 #include "misc.h"
 #include "getvt.h"
@@ -26,7 +27,15 @@ static int
 open_vt (int vtno)
 {
 	char *vtname = g_strdup_printf ("/dev/tty%d", vtno);
-	int fd = open (vtname, O_RDWR);
+	int fd;
+	do {
+		errno = 0;
+		fd = open (vtname, O_RDWR
+#ifdef O_NOCTTY
+			   |O_NOCTTY
+#endif
+			   , 0);
+	} while G_UNLIKELY (errno == EINTR);
 	g_free (vtname);
 	return fd;
 }
@@ -40,18 +49,25 @@ get_free_vt (int *vtfd)
 
 	*vtfd = -1;
 
-	fd = open ("/dev/console", O_WRONLY, 0);
+	do {
+		errno = 0;
+		fd = open ("/dev/console", O_WRONLY
+#ifdef O_NOCTTY
+			   |O_NOCTTY
+#endif
+			   , 0);
+	} while G_UNLIKELY (errno == EINTR);
 	if (fd < 0)
 		return -1;
 
 	if ((ioctl(fd, VT_OPENQRY, &vtno) < 0) || (vtno == -1)) {
-		IGNORE_EINTR (close (fd));
+		VE_IGNORE_EINTR (close (fd));
 		return -1;
 	}
 
 	fdv = open_vt (vtno);
 	if (fdv < 0) {
-		IGNORE_EINTR (close (fd));
+		VE_IGNORE_EINTR (close (fd));
 		return -1;
 	}
 
@@ -81,7 +97,7 @@ get_free_vt (int *vtfd)
 
 cleanup:
 	for (li = to_close_vts; li != NULL; li = li->next) {
-		IGNORE_EINTR (close (GPOINTER_TO_INT (li->data)));
+		VE_IGNORE_EINTR (close (GPOINTER_TO_INT (li->data)));
 	}
 	return vtno;
 }
@@ -101,23 +117,56 @@ gdm_get_empty_vt_argument (int *fd, int *vt)
 		return g_strdup_printf ("vt%d", *vt);
 }
 
+/* change to an existing vt */
 void
 gdm_change_vt (int vt)
 {
-	char *cmd;
+	int fd;
 	if (vt < 0)
 		return;
-	/* kind of hack, we could do it ourselves if we weren't lazy */
-	/* FIXME: we should really do the vt thing ourselves */
-	if (access ("/usr/bin/chvt", X_OK) == 0)
-		cmd = g_strdup_printf ("/usr/bin/chvt %d", vt);
-	else if (access ("/bin/chvt", X_OK) == 0)
-		cmd = g_strdup_printf ("/bin/chvt %d", vt);
-	else
-		cmd = NULL;
-	if (cmd != NULL)
-		system (cmd);
-	g_free (cmd);
+
+	do {
+		errno = 0;
+		fd = open ("/dev/console", O_WRONLY
+#ifdef O_NOCTTY
+			   |O_NOCTTY
+#endif
+			   , 0);
+	} while G_UNLIKELY (errno == EINTR);
+	if (fd < 0)
+		return;
+
+	ioctl (fd, VT_ACTIVATE, vt);
+	ioctl (fd, VT_WAITACTIVE, vt);
+
+	VE_IGNORE_EINTR (close (fd));
+}
+
+int
+gdm_get_cur_vt (void)
+{
+	struct vt_stat s;
+	int fd;
+
+	do {
+		errno = 0;
+		fd = open ("/dev/console", O_WRONLY
+#ifdef O_NOCTTY
+			   |O_NOCTTY
+#endif
+			   , 0);
+	} while G_UNLIKELY (errno == EINTR);
+	if (fd < 0)
+		return -1;
+
+	ioctl (fd, VT_GETSTATE, &s);
+
+	VE_IGNORE_EINTR (close (fd));
+
+	/* debug */
+	printf ("current_Active %d\n", (int)s.v_active);
+
+	return s.v_active;
 }
 
 #else /* here this is just a stub, we don't know how to do this outside
@@ -137,4 +186,10 @@ gdm_change_vt (int vt)
 	return;
 }
 
+int
+gdm_get_cur_vt (void)
+{
+	return -1;
+}
+
 #endif
