@@ -19,7 +19,7 @@
 #ifndef MISC_H
 #define MISC_H
 
-#include <gnome.h>
+void	gdm_set_theme			(const char *theme_name);
 
 #endif /* MISC_H */
 
